/*  Streaming functions for the BME280 environment sensor.

    Copyright (C) 2019 Wolfgang Reissenberger <sterne-jaeger@t-online.de>

    This application is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
*/
#include <BME280.h>
#include <BME280I2C.h>
#include <EnvironmentCalculations.h>
#include <Wire.h>
BME280I2C bme;

struct {
  bool status;
  float temperature;
  float pressure;
  float humidity;
} bmeData;


void updateBME() {
  if (bmeData.status || (bmeData.status = bme.begin())){
  float temp(NAN), hum(NAN), pres(NAN);

    BME280::TempUnit tempUnit(BME280::TempUnit_Celsius);
    BME280::PresUnit presUnit(BME280::PresUnit_hPa);
    bme.read(pres, temp, hum, tempUnit, presUnit);    
    bmeData.temperature = temp;
    bmeData.pressure    = pres;
    bmeData.humidity    = hum;
  }
}

void serializeBME(JsonDocument &doc) {

  JsonObject data = doc.createNestedObject("BME280");
  data["init"] = bmeData.status;

  if (bmeData.status) {
    data["Temp"] = bmeData.temperature;
    data["Pres"] = bmeData.pressure;
    data["Hum"] = bmeData.humidity;
  }
}
